//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
//
// Random Sentence Generator template
//
// University of Delaware
// CISC181, spring, 2010
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include "RSG.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// set seed based on time so no two games are alike

void init_random()
{
  struct timeval tp;

  gettimeofday(&tp, NULL);
  srand48(tp.tv_sec);
}

//----------------------------------------------------------------------------

// random float in range [lower, upper]

double uniform_random(double lower, double upper)
{
  double result;
  double range_size;

  range_size = upper - lower;
  result = range_size * drand48();
  result += lower;

  return result;
}

//----------------------------------------------------------------------------

// random int in range [lower, upper]

int uniform_random_int(int lower, int upper)
{
  return (int) round(uniform_random(lower, upper));
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// convert list of strings to single string via concatenation

string Sentence2String(Sentence & S)
{
  string s;

  s.clear();

  for (Sentence::iterator i = S.begin(); i != S.end(); i++) {
    if ((*i)[0] != ',')  // insert space unless next string starts with comma
      s += " ";
    s += *i;             // concatenate next string
  }

  return s;
}

//----------------------------------------------------------------------------

// print a single rule (one line of a rule set)

void PrintRule(Rule & R)
{
  cout << "- ";
  for (Rule::iterator i = R.begin(); i != R.end(); i++)
    cout << *i << " ";
  cout << endl;
}

//----------------------------------------------------------------------------

// print a whole rule set: the non-terminal and all of the associated rules

void PrintRuleSet(string & NonTerm, RuleSet & RS)
{ 
  cout << NonTerm << endl;

  for (int i = 0; i < RS.size(); i++) 
    PrintRule(RS[i]);
}

//----------------------------------------------------------------------------

// print a whole grammar: every rule set

void PrintGrammar(Grammar & G)
{
  int index = 0;
  string s;
  Grammar::iterator i;

  for (i = G.begin(); i != G.end(); i++) {
    cout << "** " << index++ << " **\n";
    s = i->first;  // for some reason can't just drop i->first into PrintRuleSet()
    PrintRuleSet(s, i->second);
  } 
}

//----------------------------------------------------------------------------

// is this string delimited by < and > ?

bool IsNonTerminal(string & s)
{
  if (s.size() > 0 && s[0] == NONTERMINAL_START_CHAR && s[s.length() - 1] == NONTERMINAL_END_CHAR)
    return true;
  else
    return false;
}

//----------------------------------------------------------------------------

// directly instantiate grammar given on project #3 page--without reading .g file.
// you can use this to test the output of your ParseGrammar() on that file, or 
// simply testing ExpandRandomly() on it without having to get ParseGrammar() working
// first

void CreateSampleGrammar(Grammar &G)
{
  Rule R;
  RuleSet RS;

  G.clear();

  // <start> rule set

  RS.clear();

  R.clear();
  R.push_back("The");
  R.push_back("<noun-phrase>");
  R.push_back("<verb>");
  R.push_back("<location>");
  RS.push_back(R);

  G["<start>"] = RS;

  // <noun-phrase> rule set

  RS.clear();
  
  R.clear();
  R.push_back("<noun>");
  RS.push_back(R);

  R.clear();
  R.push_back("<adjective>");
  R.push_back("<noun-phrase>");
  RS.push_back(R);

  G["<noun-phrase>"] = RS;

  // <adjective> rule set

  RS.clear();
  
  R.clear();
  R.push_back("large");
  RS.push_back(R);

  R.clear();
  R.push_back("brown");
  RS.push_back(R);

  G["<adjective>"] = RS;

  // <noun> rule set

  RS.clear();
  
  R.clear();
  R.push_back("cat");
  RS.push_back(R);

  R.clear();
  R.push_back("dog");
  RS.push_back(R);

  R.clear();
  R.push_back("table");
  RS.push_back(R);

  G["<noun>"] = RS;

  // <verb> rule set

  RS.clear();
  
  R.clear();
  R.push_back("jumps");
  RS.push_back(R);

  R.clear();
  R.push_back("sits");
  RS.push_back(R);

  G["<verb>"] = RS;

  // <location> rule set

  RS.clear();
  
  R.clear();
  R.push_back("under");
  R.push_back("the");
  R.push_back("stairs");
  RS.push_back(R);

  R.clear();
  R.push_back("wherever");
  R.push_back("it");
  R.push_back("wants");
  RS.push_back(R);

  G["<location>"] = RS;

}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
